﻿namespace UltraGridSpecialSelection
{
    using System;
    using System.Data;
    using System.Diagnostics;
    using System.Linq;
    using System.Windows.Forms;
    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        private UltraGridCell activeCellCached = null;
        public Form1()
        {
            InitializeComponent();
            this.Load += Form1_Load;
            this.ultraGrid1.InitializeLayout += UltraGrid1_InitializeLayout;
            this.ultraGrid1.BeforeSelectChange += UltraGrid1_BeforeSelectChange;
            this.ultraGrid1.AfterSelectChange += UltraGrid1_AfterSelectChange;
            this.ultraGrid1.KeyDown += UltraGrid1_KeyDown;
        }

        private void UltraGrid1_KeyDown(object sender, KeyEventArgs e)
        {
            //  If cached cell in not null most probably we have some cells
            // selection so go on
            if(this.activeCellCached != null)
            {
                //  check if there are selected cells. If so go on
                var selection = this.ultraGrid1.Selected;
                if(selection.Cells.Count < 1)
                {
                    return;
                }

                //  If Shift key is down go on
                if(e.Shift == true)
                {
                    //  get upper-right, upper-left, down-left and down-right cells
                    //  as you will need all these while processing Shift+Arrow keys
                    var selectedCells = selection.Cells;
                    var ulCell = selectedCells
                        .Cast<UltraGridCell>()
                        .OrderBy(c => c.Row.Index)
                        .OrderBy(c => c.Column.Index)
                        .FirstOrDefault();

                    var urCell = selectedCells
                        .Cast<UltraGridCell>()
                        .OrderBy(c => c.Row.Index)
                        .OrderByDescending(c => c.Column.Index)
                        .FirstOrDefault();

                    var blCell = selectedCells
                        .Cast<UltraGridCell>()
                        .OrderByDescending(c => c.Row.Index)
                        .OrderBy(c => c.Column.Index)
                        .FirstOrDefault();

                    var brCell = selectedCells
                        .Cast<UltraGridCell>()
                        .OrderByDescending(c => c.Row.Index)
                        .OrderByDescending(c => c.Column.Index)
                        .FirstOrDefault();

                    switch(e.KeyCode)
                    {
                        #region Up
                        case Keys.Up:
                            {
                                //  Set Handled to true, so nothing else will happen but our logic
                                e.Handled = true;

                                //  Get the index of bottom row of the current selection
                                var rowIndex = blCell.Row.Index;

                                //  Get the indexes of start end end columns
                                var startCol = blCell.Column.Index;
                                var endCol = brCell.Column.Index;

                                //  In MS Excel if the active cell is above the selection
                                //  and someone presses Shift+UpArrow the bottom row gets
                                //  cleared from selection; if the active cell is in the
                                //  bottom row of the selection or is under this row the
                                //  selection extends with one row up. The same logic is
                                //  implemented bellow
                                if(this.activeCellCached.Row.Index >= rowIndex)
                                {
                                    //  If cached cell is bellow or at bottom row we have
                                    //  to extend the selection upward, so get the index
                                    //  of top row
                                    rowIndex = ulCell.Row.Index;

                                    //  if this is the topmost row of the grid do nothing
                                    if(rowIndex == 0)
                                    {
                                        return;
                                    }

                                    //  Toggle selection of each cell in the row above the
                                    //  top row
                                    for(int col = startCol; col <= endCol; col++)
                                    {
                                        var cell = this.ultraGrid1.Rows[rowIndex - 1].Cells[col];
                                        cell.Selected = !cell.Selected;
                                    }
                                }
                                else
                                {
                                    //  If cached cell above bottom row we need to clear
                                    //  the selection of the cells in bottom row
                                    for(int col = startCol; col <= endCol; col++)
                                    {
                                        var cell = this.ultraGrid1.Rows[rowIndex].Cells[col];
                                        cell.Selected = !cell.Selected;
                                    }
                                }

                                break;
                            }
                        #endregion // Up
                        #region Down
                        case Keys.Down:
                            {
                                //  Check the comments in Keys.Up case
                                e.Handled = true;
                                var rowIndex = ulCell.Row.Index;
                                var startCol = ulCell.Column.Index;
                                var endCol = urCell.Column.Index;

                                if(this.activeCellCached.Row.Index <= rowIndex)
                                {
                                    rowIndex = blCell.Row.Index;
                                    if(rowIndex == this.ultraGrid1.Rows.Count - 1)
                                    {
                                        return;
                                    }

                                    for(int col = startCol; col <= endCol; col++)
                                    {
                                        var cell = this.ultraGrid1.Rows[rowIndex + 1].Cells[col];
                                        cell.Selected = !cell.Selected;
                                    }
                                }
                                else
                                {
                                    for(int col = startCol; col <= endCol; col++)
                                    {
                                        var cell = this.ultraGrid1.Rows[rowIndex].Cells[col];
                                        cell.Selected = !cell.Selected;
                                    }
                                }
                                break;
                            }
                        #endregion // Down
                        #region Left
                        case Keys.Left:
                            {
                                //  Check the comments in Keys.Up case
                                e.Handled = true;
                                var colIndex = urCell.Column.Index;
                                var startRow = urCell.Row.Index;
                                var endRow = brCell.Row.Index;

                                if(this.activeCellCached.Column.Index >= colIndex)
                                {
                                    colIndex = ulCell.Column.Index;
                                    if(colIndex == 0)
                                    {
                                        return;
                                    }

                                    for(int row = startRow; row <= endRow; row++)
                                    {
                                        var cell = this.ultraGrid1.Rows[row].Cells[colIndex - 1];
                                        cell.Selected = !cell.Selected;
                                    }
                                }
                                else
                                {
                                    for(int row = startRow; row <= endRow; row++)
                                    {
                                        var cell = this.ultraGrid1.Rows[row].Cells[colIndex];
                                        cell.Selected = !cell.Selected;
                                    }
                                }

                                break;
                            }
                        #endregion // Left
                        #region Right
                        case Keys.Right:
                            {
                                //  Check the comments in Keys.Up case
                                e.Handled = true;
                                var colIndex = ulCell.Column.Index;
                                var startRow = ulCell.Row.Index;
                                var endRow = blCell.Row.Index;

                                if(this.activeCellCached.Column.Index <= colIndex)
                                {
                                    colIndex = urCell.Column.Index;
                                    if(colIndex == this.ultraGrid1.DisplayLayout.Bands[0].Columns.Count - 1)
                                    {
                                        return;
                                    }

                                    for(int row = startRow; row <= endRow; row++)
                                    {
                                        var cell = this.ultraGrid1.Rows[row].Cells[colIndex + 1];
                                        cell.Selected = !cell.Selected;
                                    }
                                }
                                else
                                {
                                    for(int row = startRow; row <= endRow; row++)
                                    {
                                        var cell = this.ultraGrid1.Rows[row].Cells[colIndex];
                                        cell.Selected = !cell.Selected;
                                    }
                                }

                                break;
                            } 
                            #endregion // Right
                    }
                }
            }
        }

        private void Print(SelectedCellsCollection selectedCells)
        {
            foreach(var cell in selectedCells)
            {
                Debug.WriteLine($"Row{cell.Row.Index}:Col{cell.Column.Index}");
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //  Add some dummy data to the grid
            var data = this.GetData();
            this.ultraGrid1.SetDataBinding(data, data.TableName);
        }

        private void UltraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            //  Set up the grid so the user can select cells while dragging
            e.Layout.Override.CellClickAction = CellClickAction.CellSelect;
        }

        private void UltraGrid1_BeforeSelectChange(object sender, BeforeSelectChangeEventArgs e)
        {
            //  If the new selection is of cells get a reference to the first
            //  selected cell and assign it to the cached active cell field
            if(e.NewSelections.Cells.Count == 1)
            {
                this.activeCellCached = this.ultraGrid1.ActiveCell;
            }
        }

        private void UltraGrid1_AfterSelectChange(object sender, AfterSelectChangeEventArgs e)
        {
            //  If cashed active cell is not null set the grid active cell to
            //  refer to this cached cell value
            if(this.activeCellCached != null)
            {
                this.ultraGrid1.ActiveCell = this.activeCellCached;
            }
        }

        private DataTable GetData(int rows = 20, int cols = 5)
        {
            DataTable dt = new DataTable();

            for(int col = 0; col < cols; col++)
            {
                dt.Columns.Add(string.Format("Column #{0}", col + 1));
            }

            for(int row = 0; row < rows; row++)
            {
                DataRow addRow = dt.NewRow();
                foreach(DataColumn column in dt.Columns)
                {
                    addRow[column] = string.Format("{0}.{1}", row + 1, column.Ordinal + 1);
                }

                dt.Rows.Add(addRow);
            }

            return dt;
        }
    }
}
